#include-once
#include <String.au3>

; #INDEX# =======================================================================================================================
; Title .........: TBAutomation
; Description ...: The TwistedBrush Automation client functions that interface with TwistedBrush Pro Studio Automation server.
; Author(s) .....: Ken Carlino
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;TBAutomationInit
;TBAutomationClose
;TBAutomationCmdStart
;TBAutomationCmdDone
;TBAutomationCmdIsLayer
;TBAutomationCmdGotoLayer
;TBAutomationCmdSaveLayerToPNG
;TBAutomationCmdLoadPNGToLayer
;TBAutomationCmdCreateLayer
;TBAutomationCmdClearPage
;TBAutomationCmdScriptPlaytback
;TBAutomationSendCommand
;TBAutomationReadLine
; ===============================================================================================================================

; Stardard error codes
Global Const $TBAUTOMATION_GENERAL_FAILURE = 1 				; General failure
Global Const $TBAUTOMATION_NO_SERVER = 2 					; Automation server not found
Global Const $TBAUTOMATION_CONNECTION_FAILURE = 3 			; Unable to open a connection to the Automation server
Global Const $TBAUTOMATION_NO_CONNECTION = 4 				; No current connection to the Automation server

; Global vars
Global $TBAutomationSocket = -1

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationInit
; Description ...: Iniitalize the TBAutomation client.
; Syntax.........: TBAutomationInit()
; Parameters ....: 
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_NO_SERVER, returns False
;                  Error: @error set to $TBAUTOMATION_CONNECTION_FAILURE, returns False 
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: TBAutomationClose
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationInit()
	If WinExists("TwistedBrush Automation") == 0 Then
		Return SetError($TBAUTOMATION_NO_SERVER, 0, False)
	EndIf	

	TCPStartup()

	$TBAutomationSocket = TCPConnect("127.0.0.1", 8124)
	If $TBAutomationSocket = -1 Then
		Return SetError($TBAUTOMATION_CONNECTION_FAILURE, 0, False)
	EndIf
	Return SetError(0, 0, True)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationClose
; Description ...: Closes the TBAutomation client.
; Syntax.........: TBAutomationClose()
; Parameters ....: 
; Return values .: Success - True
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: TBAutomationInit
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationClose()
	TCPCloseSocket($TBAutomationSocket) 
	$TBAutomationSocket = -1
	TCPShutdown()
	Return SetError(0, 0, True)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdStart
; Description ...: Send the Start command. Tells the server to get ready to receive commands.
; Syntax.........: TBAutomationCmdStart()
; Parameters ....: 
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: TBAutomationDone
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdStart()
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("start")
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdDone
; Description ...: Send the Done command. Tells the server that no more commands will be sent on this connection.
; Syntax.........: TBAutomationCmdDone()
; Parameters ....: 
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: TBAutomationStart
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdDone()
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("done")
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdIsLayer
; Description ...: Send the IsLayer command. Asks the server if the layer exists
; Syntax.........: TBAutomationCmdIsLayer($layerNumber)
; Parameters ....: $layerNumber - The TwistedBrush layer number for the current page
; Return values .: Exists - True
;                  Does not exist - False 					
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: 
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdIsLayer($layerNumber)
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("islayer," & $layerNumber)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdGotoLayer
; Description ...: Send the GotoLayer command. Tells the server to switch to the specific layer.
; Syntax.........: TBAutomationCmdGotoLayer($layerNumber)
; Parameters ....: $layerNumber - The TwistedBrush layer number for the current page
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: 
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdGotoLayer($layerNumber)
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("gotolayer," & $layerNumber)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdSaveLayerToPNG
; Description ...: Send the SaveLayerToPNG command. Tells the server to save the specific layer to a PNG file
; Syntax.........: TBAutomationCmdSaveLayerToPNG($layerNumber, $fileName)
; Parameters ....: $layerNumber - The TwistedBrush layer number for the current page
;                  $fileName - The filename (full paht) 
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: 
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdSaveLayerToPNG($layerNumber, $fileName)
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("savelayertopng," & $layerNumber & "," & $fileName)
EndFunc


; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdLoadPNGToLayer
; Description ...: Send the LoadPNGToLayer command. Tells the server to load PNG imge into the specific layer
; Syntax.........: TBAutomationCmdLoadPNGToLayer($layerNumber, $fileName, $resizeFlag)
; Parameters ....: $layerNumber - The TwistedBrush layer number for the current page
;                  $fileName - The filename (full paht) 
;                  $resizeFlag - True - resize the page if needed. False - will not resize the page
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: 
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdLoadPNGToLayer($layerNumber, $fileName, $resizeFlag)
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("loadpngtolayer," & $layerNumber & "," & $fileName & "," & $resizeFlag)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdCreateLayer
; Description ...: Send the CreateLayer command. Tells the server to create the specific layer.
; Syntax.........: TBAutomationCmdCreateLayer($layerNumber)
; Parameters ....: $layerNumber - The TwistedBrush layer number for the current page
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: 
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdCreateLayer($layerNumber)
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("createlayer," & $layerNumber)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdClearPage
; Description ...: Send the ClearPage command. Tells the server to clear the current page.
; Syntax.........: TBAutomationCmdClearPage()
; Parameters ....: 
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: 
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdClearPage()
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("clearpage")
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationCmdScriptPlayback
; Description ...: Send the ScriptPlayback command. Tells the server to playback the specified script
; Syntax.........: TBAutomationCmdScriptPlayback($scriptName)
; Parameters ....: $scriptName - The TwistedBrush script name
; Return values .: Success - True
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......:
; Related .......: 
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationCmdScriptPlayback($scriptName)
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	return TBAutomationSendCommand("scriptplayback," & $scriptName)
EndFunc


; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationSendCommand
; Description ...: Sends a command to the automation server
; Syntax.........: TBAutomationSendCommand($command)
; Parameters ....: command - The automation command to send to the server
; Return values .: Success - Returns the result string from the server
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False 
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......: Normally just internally used. Use the Automatcion command specific functions instead.
; Related .......: TBAutomationReadLine
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationSendCommand($command)
	Local $result
	
	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	TCPSend($TBAutomationSocket, $command & @CRLF)

	If @error Then
		Return SetError($TBAUTOMATION_GENERAL_FAILURE, 0, False)
		TBAutomationClose()
	EndIf

	$result = TBAutomationReadLine(60000)
	If $result == "true" Then
		$result = true
	ElseIf $result == "false" Then	
		$result = false
	EndIf	
		
	return $result
EndFunc


; #FUNCTION# ====================================================================================================================
; Name...........: TBAutomationReadLine
; Description ...: Reads one result line from the automation server
; Syntax.........: TBAutomationReadLine($timeout)
; Parameters ....: timeout - How long in millisecs to wait for a response
; Return values .: Success - returns the result string sent from the server.
;                  Error: @error set to $TBAUTOMATION_GENERAL_FAILURE, returns False
;                  Error: @error set to $TBAUTOMATION_NO_CONNECTION, returns False 
; Author ........: Ken Carlino
; Modified.......: 
; Remarks .......: Normally just internally used. Use the Automatcion command specific functions instead.
; Related .......: TBAutomationSendCommand
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func TBAutomationReadLine($timeout)
	local $dataLine
	local $dataNew
	local $dataArray
	local $readMore = True

	If $TBAutomationSocket == -1 Then
		Return SetError($TBAUTOMATION_NO_CONNECTION, 0, False)
	EndIf	

	While $readMore	And $timeout > 0
		$dataNew = TCPRecv($TBAutomationSocket, 4096)
		if $dataNew <> "" Then
			$dataLine &= $dataNew
			if StringInstr($dataLine, @CRLF) then
				$dataArray = _StringExplode($dataLine, @CRLF)
				return $dataArray[0]
			EndIf
		EndIf
		Sleep(10)
		$timeout -= 10
	WEnd
	Return SetError($TBAUTOMATION_GENERAL_FAILURE, 0, False)
EndFunc




